# -*- coding: UTF-8 -*-

# pictureplayer2 E2
#
#  Coded by bobo71 (c) 2014
#  Support: www.dreambox-tools.info
#
#  All Files of this Software are licensed under the Creative Commons
#  Attribution-NonCommercial-ShareAlike 3.0 Unported
#  License if not stated otherwise in a Files Head. To view a copy of this license, visit
#  http://creativecommons.org/licenses/by-nc-sa/3.0/ or send a letter to Creative
#  Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.

#  Additionally, this plugin may only be distributed and executed on hardware which
#  is licensed by Dream Multimedia GmbH.

#  This plugin is NOT free software. It is open source, you are allowed to
#  modify it (if you keep the license), but it may not be commercially
#  distributed other than under the conditions noted above.
#  This applies to the source code as a whole as well as to parts of it, unless
#  explicitely stated otherwise.

# based on original pictureplayer 


from enigma import ePicLoad, eRect, eSize, eTimer, getDesktop, gPixmapPtr, eListboxPythonMultiContent

from Screens.Screen import Screen

from Plugins.Plugin import PluginDescriptor

from Components.Pixmap import Pixmap, MovingPixmap
from Components.ActionMap import ActionMap
from Components.Harddisk import harddiskmanager
from Components.Sources.StaticText import StaticText
from Components.FileList import FileList, FileEntryComponent
from Components.AVSwitch import AVSwitch
from Components.Sources.List import List
from Components.ConfigList import ConfigListScreen
from Components.CoverCollection import VideoDBPictureBox, CoverCollection
from Components.config import config, ConfigSubsection, ConfigInteger, ConfigSelection, ConfigText, ConfigEnableDisable, getConfigListEntry, configfile
from Screens.MessageBox import MessageBox
from Tools.Directories import resolveFilename, pathExists, fileExists, SCOPE_MEDIA, createDir

from PIL import Image, ImageColor
from os import path as os_path, listdir as os_listdir, remove as os_remove

# thread
from Tools.BoundFunction import boundFunction
from twisted.internet.threads import deferToThread

from re import compile as re_compile

from Tools.Directories import SCOPE_CURRENT_SKIN

from enigma import RT_HALIGN_LEFT, eServiceReference, eServiceCenter, gFont
from Tools.LoadPixmap import LoadPixmap

import SelectPath
from myLogger import LOG
from jobcenter import fixAspectRatio

RESOLUTIONx = getDesktop(0).size().width()

# override Filelist
# not show ""."" startswitch directories expect /.Thumbnail
class pp2FileList(FileList):

	def changeDir(self, directory, select = None):
		self.list = []
		# if we are just entering from the list of mount points:
		if self.current_directory is None:
			if directory and self.showMountpoints:
				self.current_mountpoint = self.getMountpointLink(directory)
			else:
				self.current_mountpoint = None
		self.current_directory = directory
		directories = []
		files = []

		if directory is None and self.showMountpoints: # present available mountpoints
			for p in harddiskmanager.getMountedPartitions():
				path = os_path.join(p.mountpoint, "")
				if path not in self.inhibitMounts and not self.inParentDirs(path, self.inhibitDirs):
					self.list.append(FileEntryComponent(name = p.description, absolute = path, isDir = True))
			files = [ ]
			directories = [ ]
		elif directory is None:
			files = [ ]
			directories = [ ]
		elif self.useServiceRef:
			root = eServiceReference("2:0:1:0:0:0:0:0:0:0:" + directory)
			if self.additional_extensions:
				root.setName(self.additional_extensions)
			serviceHandler = eServiceCenter.getInstance()
			list = serviceHandler.list(root)

			while 1:
				s = list.getNext()
				if not s.valid():
					del list
					break
				if s.flags & s.mustDescent:
					directories.append(s.getPath())
				else:
					files.append(s)
			directories.sort()
			files.sort()
			
		else:
			if fileExists(directory):
				try:
					files = os_listdir(directory)
				except:
					files = []
				files.sort()
				tmpfiles = files[:]
				for x in tmpfiles:
					if os_path.isdir(directory + x):
						directories.append(directory + x + "/")
						files.remove(x)
		if directory is not None and self.showDirectories and not self.isTop:
			if directory == self.current_mountpoint and self.showMountpoints:
				self.list.append(FileEntryComponent(name = "<" +_("List of Storage Devices") + ">", absolute = None, isDir = True))
			elif (directory != "/") and not (self.inhibitMounts and self.getMountpoint(directory) in self.inhibitMounts):
				self.list.append(FileEntryComponent(name = "<" +_("Parent Directory") + ">", absolute = '/'.join(directory.split('/')[:-2]) + '/', isDir = True))

		if self.showDirectories:
			for x in directories:
				if not (self.inhibitMounts and self.getMountpoint(x) in self.inhibitMounts) and not self.inParentDirs(x, self.inhibitDirs):
					name = x.split('/')[-2]
					if not name.startswith('.'):
						self.list.append(FileEntryComponent(name = name, absolute = x, isDir = True))

		if self.showFiles:
			for x in files:
				if self.useServiceRef:
					path = x.getPath()
					name = path.split('/')[-1]
				else:
					path = directory + x
					name = x

				if (self.matchingPattern is None) or re_compile(self.matchingPattern).search(path):
					self.list.append(FileEntryComponent(name = name, absolute = x , isDir = False))

		if self.showMountpoints and len(self.list) == 0:
			self.list.append(FileEntryComponent(name = _("nothing connected"), absolute = None, isDir = False))
		self.l.setList(self.list)

		if select is not None:
			i = 0
			self.moveToIndex(0)
			for x in self.list:
				p = x[0][0]

				if isinstance(p, eServiceReference):
					p = p.getPath()

				if p == select:
					self.moveToIndex(i)
				i += 1


def getScale():
	return AVSwitch().getFramebufferScale()


def setPixmap(dest, ptr, scaleSize, aspectRatio):
	if scaleSize.isValid() and aspectRatio.isValid():
		pic_scale_size = ptr.size().scale(scaleSize, aspectRatio)
		pic_size = ptr.size()
		pic_width = pic_size.width()
		pic_height = pic_size.height()

		dest_size = dest.getSize()
		dest_width = dest_size.width()
		dest_height = dest_size.height()

		pic_scale_width = pic_scale_size.width()
		pic_scale_height = pic_scale_size.height()

		#print "PicturePlayer2]pic size %dx%d" %(pic_width, pic_height)
		#print "PicturePlayer2]pic scale size %dx%d" %(pic_scale_width, pic_scale_height)
		#print "PicturePlayer2]dest area size %dx%d" %(dest_width, dest_height)

		if pic_scale_width == dest_width: # v center
			dest_rect = eRect(0, (dest_height - pic_scale_height) / 2, pic_scale_width, pic_scale_height)
		else: # h center
			dest_rect = eRect((dest_width - pic_scale_width) / 2, 0, pic_scale_width, pic_scale_height)

		#print "PicturePlayer2]dest rect left->%s top->%s width->%s heigth->%s" %(dest_rect.left(), dest_rect.top(), dest_rect.width(), dest_rect.height())

		dest.instance.setScale(1)
		dest.instance.setScaleDest(dest_rect)
	else:
		print "PicturePlayer2]no scale!"
		dest.instance.setScale(0)
	dest.instance.setPixmap(ptr)
	
def resolveThumbOutPath(picpath):
	isok=False
	path, picname=os_path.split(picpath)
	thumbdir=''.join((path, '/.Thumbnails/'))
	bgcolor = config.pic2.bgcolor.value
	picpathout=''.join((thumbdir, bgcolor[1:], '_', picname))
	if not os_path.exists(thumbdir):
		createDir(thumbdir)
	try: 
		thumbsindirectory=os_listdir(thumbdir)
	except:
		thumbsindirectory=[]
	for pic in iter(thumbsindirectory):
		if picname in pic: # pic matach
			#print 'pic->',pic,' picpathout->%s'  %picpathout
			if bgcolor[1:] in pic and os_path.exists(picpathout): # color match
				LOG('resolveThumbOutPath] is on cache, check timestamp...')
				if os_path.getmtime(picpath) < os_path.getmtime(picpathout):
					LOG('resolveThumbOutPath]timestamp ok')
					isok=True
					break
				else:
					LOG('resolveThumbOutPath]timestamp not ok, rebuild thumb!')
					os_remove(picpathout)
					break
			else: # dont match color
				for outsortedpic in [ pic for pic in thumbsindirectory if picname in pic ]:
					outsortedpicfilename=''.join((thumbdir, outsortedpic))
					LOG('resolveThumbOutPath]remove obsolete thumb->%s' %outsortedpicfilename)
					if os_path.exists(outsortedpicfilename):
						os_remove(outsortedpicfilename)
	return (picpathout, isok)

class picshow2(Screen):

	if RESOLUTIONx>1800:
		skin = """
			<screen name="picshow2.02" position="center,center" size="840,660" title="PicturePlayer2" >
				<ePixmap pixmap="skin_default/buttons/red.png" position="0,0" size="210,60" alphatest="on" />
				<ePixmap pixmap="skin_default/buttons/green.png" position="210,0" size="210,60" alphatest="on" />
				<ePixmap pixmap="skin_default/buttons/yellow.png" position="420,0" size="210,60" alphatest="on" />
				<ePixmap pixmap="skin_default/buttons/blue.png" position="630,0" size="210,60" alphatest="on" />
				<ePixmap pixmap="Default-HD/icons/info.png" position="2,65" size="54,37" alphatest="on" zPosition="5"/>
				<widget source="key_red" render="Label" position="0,0" zPosition="1" size="210,60" font="Regular;30" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" />
				<widget source="key_green" render="Label" position="210,0" zPosition="1" size="210,60" font="Regular;30" halign="center" valign="center" backgroundColor="#1f771f" transparent="1" />
				<widget source="key_yellow" render="Label" position="420,0" zPosition="1" size="210,60" font="Regular;30" halign="center" valign="center" backgroundColor="#a08500" transparent="1" />
				<widget source="key_blue" render="Label" position="630,0" zPosition="1" size="210,60" font="Regular;30" halign="center" valign="center" backgroundColor="#18188b" transparent="1" />
				<widget source="key_info" render="Label" position="60,65" zPosition="1" size="210,60" font="Regular;30" halign="center" valign="center" backgroundColor="#18188b" transparent="1" />
				<widget source="label" render="Label" position="8,143" size="525,250" font="Regular;27" backgroundColor="#25062748" transparent="1"  />
				<widget name="thn" position="540,60" size="270,240" />
				<widget name="filelist" position="8,310" zPosition="2" size="825,345" scrollbarMode="showOnDemand" />
			</screen>"""
	else:
		skin = """
			<screen name="picshow2.02" position="center,center" size="560,440" title="PicturePlayer2" >
				<ePixmap pixmap="skin_default/buttons/red.png" position="0,0" size="140,40" alphatest="on" />
				<ePixmap pixmap="skin_default/buttons/green.png" position="140,0" size="140,40" alphatest="on" />
				<ePixmap pixmap="skin_default/buttons/yellow.png" position="280,0" size="140,40" alphatest="on" />
				<ePixmap pixmap="skin_default/buttons/blue.png" position="420,0" size="140,40" alphatest="on" />
				<ePixmap pixmap="Default-HD/icons/info.png" position="1,45" size="54,37" alphatest="on" zPosition="5"/>
				<widget source="key_red" render="Label" position="0,0" zPosition="1" size="140,40" font="Regular;20" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" />
				<widget source="key_green" render="Label" position="140,0" zPosition="1" size="140,40" font="Regular;20" halign="center" valign="center" backgroundColor="#1f771f" transparent="1" />
				<widget source="key_yellow" render="Label" position="280,0" zPosition="1" size="140,40" font="Regular;20" halign="center" valign="center" backgroundColor="#a08500" transparent="1" />
				<widget source="key_blue" render="Label" position="420,0" zPosition="1" size="140,40" font="Regular;20" halign="center" valign="center" backgroundColor="#18188b" transparent="1" />
				<widget source="key_info" render="Label" position="40,45" zPosition="1" size="140,40" font="Regular;20" halign="center" valign="center" backgroundColor="#18188b" transparent="1" />
				<widget source="label" render="Label" position="5,55" size="350,140" font="Regular;19" backgroundColor="#25062748" transparent="1"  />
				<widget name="thn" position="360,40" size="180,160" />
				<widget name="filelist" position="5,205" zPosition="2" size="550,230" scrollbarMode="showOnDemand" />
			</screen>"""

	def __init__(self, session):
		Screen.__init__(self, session)

		self["actions"] = ActionMap(["OkCancelActions", "ColorActions", "DirectionActions", "EPGSelectActions"],
		{
			"cancel": self.KeyExit,
			"red": self.KeyExit,
			"green": self.KeyGreen,
			"yellow": self.KeyYellow,
			"blue": self.KeyBlue,
			"ok": self.KeyOk,
			"info" : self.infoPressed,
		}, -1)
		
		self["key_info"] = StaticText(_(""))
		self["key_red"] = StaticText(_("Close"))
		self["key_green"] = StaticText(_("Thumbnails"))
		self["key_yellow"] = StaticText("Home")
		self["key_blue"] = StaticText(_("Setup"))
		self["label"] = StaticText("")
		self["thn"] = Pixmap()

		currDir = config.pic2.lastDir.value
		if not pathExists(currDir):
			currDir = "/"
		
		self.filelist = pp2FileList(currDir, matchingPattern = "(?i)^.*\.(jpeg|jpg|jpe|png|bmp|gif)")
		self["filelist"] = self.filelist
		self["filelist"].onSelectionChanged.append(self.selectionChanged)

		self.ThumbTimer = eTimer()
		self.ThumbTimer_conn = self.ThumbTimer.timeout.connect(self.showThumb)

		self.picload = ePicLoad()
		self.picload_conn = self.picload.PictureData.connect(self.showPic)
		self.onLayoutFinish.append(self.setConf)

	def showPic(self, picInfo=""):
		ptr = self.picload.getData()
		if ptr != None:
			setPixmap(self["thn"], ptr, self._scaleSize, self._aspectRatio)
			self["thn"].show()

		text = picInfo.split('\n',1)
		self["label"].setText(text[1])
		self["key_info"].setText(_("Exif"))

	def showThumb(self):
		if not self.filelist.canDescent():
			if self.filelist.getCurrentDirectory() and self.filelist.getFilename():
				if fileExists(self.filelist.getCurrentDirectory() + self.filelist.getFilename()):
					thumbfile, isok=resolveThumbOutPath(self.filelist.getCurrentDirectory() + self.filelist.getFilename())# check for rotated and parsed Image
					if not isok:
						thumbfile=self.filelist.getCurrentDirectory() + self.filelist.getFilename()
					#print 'PicturePlayer2]showThumb]->%s'  %thumbfile
					if self.picload.getThumbnail(thumbfile) == 1:
						self.ThumbTimer.start(config.pic2.thumbDelay.value, True)

	def selectionChanged(self):
		if not self.filelist.canDescent():
			self.ThumbTimer.start(config.pic2.thumbDelay.value, True)
		else:
			self["label"].setText("")
			self["thn"].hide()
			self["key_info"].setText("")

	def KeyGreen(self):
		#if not self.filelist.canDescent():
		self.session.openWithCallback(self.callbackView, Pic_Thumb2, self.filelist.getFileList(), self.filelist.getSelectionIndex(), self.filelist.getCurrentDirectory())

	def KeyYellow(self):
		answer=config.pic2.pichome.value
		LOG('config.pic2.pichome.value-->%s' %answer)
		try:
			if answer[0] == "/":
				self["filelist"].changeDir(answer)
				#print 'Filelist]setBookmark] use bookmark .changeDir:%s' %str(answer[:-1])
		except:
			pass
			
	def infoPressed(self):
		if not self.filelist.canDescent():
			self.session.open(Pic_Exif2, self.picload.getInfo(self.filelist.getCurrentDirectory() + self.filelist.getFilename()))

	def KeyBlue(self):
		self.session.openWithCallback(self.setConf ,Pic_Setup2)

	def KeyOk(self):
		if self.filelist.canDescent():
			self.filelist.descent()
		else:
			index=self.filelist.getSelectionIndex()-1
			if index <0:
				index=0
			self.session.openWithCallback(self.callbackView, Pic_Full_View2, self.filelist.getFileList(), index, self.filelist.getCurrentDirectory())

	def setConf(self):
		self.setTitle(_("PicturePlayer2"))
		sc = getScale()
		#print "PicturePlayer2]setConf", sc[0], ' ', sc[1]
		self._aspectRatio = eSize(sc[0], sc[1])
		self._scaleSize = self["thn"].instance.size()
		#0=Width 1=Height 2=Aspect 3=use_cache 4=resize_type 5=Background(#AARRGGBB)
		params = (self._scaleSize.width(), self._scaleSize.height(), sc[0], sc[1], config.pic2.cache.value, int(config.pic2.resize.value), "#00000000")
		self.picload.setPara(params)

	def callbackView(self, val=0, ischanged=False):
		if val > 0 or ischanged:
			self.filelist.moveToIndex(val)

	def KeyExit(self):
		self.ThumbTimer.stop()
		del self.picload_conn
		del self.picload
		if self.filelist.getCurrentDirectory() is None:
			config.pic2.lastDir.value = "/"
		else:
			config.pic2.lastDir.value = self.filelist.getCurrentDirectory()
		config.pic2.save()
		self.close()
		
#------------------------------------------------------------------------------------------

class Pic_Setup2(Screen, ConfigListScreen):

	def __init__(self, session):
		Screen.__init__(self, session)
		# for the skin: first try MediaPlayerSettings, then Setup, this allows individual skinning
		self.skinName = ["PicturePlayerSetup", "Setup" ]
		self.setup_title = _("PicturePlayer 2.02 Settings")
		self.onChangedEntry = [ ]
		self.session = session

		self["actions"] = ActionMap(["SetupActions"],
			{
				"cancel": self.keyCancel,
				"green": self.closeNow,
				"save": self.closeNow,
				"ok": self.keySelect,
			}, -2)# "save": self.keySave,

		self["key_red"] = StaticText(_("Cancel"))
		self["key_green"] = StaticText(_("OK"))

		self.list = []
		ConfigListScreen.__init__(self, self.list, session = self.session, on_change = self.changedEntry)
		self.createSetup()
		self.onLayoutFinish.append(self.layoutFinished)

	def layoutFinished(self):
		self.setTitle(self.setup_title)

	def createSetup(self):
		self.list = []
		self.databasepath = getConfigListEntry(_("Select home path"), config.pic2.pichome)
		self.list.append(self.databasepath)

		self.list.append(getConfigListEntry(_("Slideshow: Interval between pictures (sec)"), config.pic2.slidetime))
		self.list.append(getConfigListEntry(_("Slideshow: Repeat"), config.pic2.loop))
		self.list.append(getConfigListEntry(_("Fullscreen: Show info line"), config.pic2.infoline))
		self.list.append(getConfigListEntry(_("Fullscreen: Text color"), config.pic2.textcolor))
		self.list.append(getConfigListEntry(_("Fullscreen: Background color"), config.pic2.bgcolor))
		self.list.append(getConfigListEntry(_("Fullscreen: Frame size"), config.pic2.framesize))
		self.list.append(getConfigListEntry(_("Thumbnails: Enable cache"), config.pic2.cache))
		self.list.append(getConfigListEntry(_("Thumbnails: Delay before loading (msec)"), config.pic2.thumbDelay))
		self.list.append(getConfigListEntry(_("Fullscreen: Fadetime between pictures (msec)"), config.pic2.fadeDelay))
		self.list.append(getConfigListEntry(_("Thumbnails: JPEG Scaling Mode (experimental"), config.pic2.resize))
		self["config"].list = self.list
		self["config"].l.setList(self.list)

	def keyLeft(self):
		ConfigListScreen.keyLeft(self)

	def keyRight(self):
		ConfigListScreen.keyRight(self)

	# for summary:
	def changedEntry(self):
		for x in self.onChangedEntry:
			x()

	def getCurrentEntry(self):
		return self["config"].getCurrent()[0]

	def getCurrentValue(self):
		return str(self["config"].getCurrent()[1].getText())

	def createSummary(self):
		from Screens.Setup import SetupSummary
		return SetupSummary

	def keySelect(self):
		cur = self["config"].getCurrent()
		if cur == self.databasepath:
			self.session.openWithCallback(self.pathSelectedpiclibrary, SelectPath.SelectPath, config.pic2.pichome.value)

	def pathSelectedpiclibrary(self, res):
		if res is not None:
			config.pic2.pichome.value = res[0]
		
	def closeNow(self):
		self.saveAll()
		configfile.save()
		self.close()
#---------------------------------------------------------------------------

class Pic_Exif2(Screen):

	skin = """
		<screen name="Pic_Exif2.02" position="center,center" size="560,360" title="Info" >
			<ePixmap pixmap="skin_default/buttons/red.png" position="0,0" size="140,40" alphatest="on" />
			<widget source="key_red" render="Label" position="0,0" zPosition="1" size="140,40" font="Regular;20" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" />
			<widget source="menu" render="Listbox" position="5,50" size="550,310" scrollbarMode="showOnDemand" selectionDisabled="1" >
				<convert type="TemplatedMultiContent">
				{
					"template": [  MultiContentEntryText(pos = (5, 5), size = (250, 30), flags = RT_HALIGN_LEFT, text = 0), MultiContentEntryText(pos = (260, 5), size = (290, 30), flags = RT_HALIGN_LEFT, text = 1)],
					"fonts": [gFont("Regular", 20)],
					"itemHeight": 30
				}
				</convert>
			</widget>
		</screen>"""

	def __init__(self, session, exiflist):
		Screen.__init__(self, session)

		self["actions"] = ActionMap(["SetupActions", "ColorActions"],
		{
			"cancel": self.close
		}, -1)

		self["key_red"] = StaticText(_("Close"))

		exifdesc = [_("filename")+':', "EXIF-Version:", "Make:", "Camera:", "Date/Time:", "Width / Height:", "Flash used:", "Orientation:", "User Comments:", "Metering Mode:", "Exposure Program:", "Light Source:", "CompressedBitsPerPixel:", "ISO Speed Rating:", "X-Resolution:", "Y-Resolution:", "Resolution Unit:", "Brightness:", "Exposure Time:", "Exposure Bias:", "Distance:", "CCD-Width:", "ApertureFNumber:"]
		list = []

		for x in range(len(exiflist)):
			if x>0:
				list.append((exifdesc[x], exiflist[x]))
			else:
				name = exiflist[x].split('/')[-1]
				list.append((exifdesc[x], name))
		self["menu"] = List(list)
		self.onLayoutFinish.append(self.layoutFinished)

	def layoutFinished(self):
		self.setTitle(_("Info"))

#----------------------------------------------------------------------------------------

T_INDEX = 0
T_FRAME_POS = 1
T_PAGE = 2
T_NAME = 3
T_FULL = 4

class Pic_Thumb2(Screen):
	def __init__(self, session, piclist, lastindex, path):

		self.textcolor = config.pic2.textcolor.value
		self.color = config.pic2.bgcolor.value
		textsize = 24
		self.spaceX = 15#35
		self.picX = 220#190
		self.spaceY = 15#35
		self.picY = 200

		size_w = getDesktop(0).size().width()
		size_h = getDesktop(0).size().height()
		self.thumbsX = size_w / (self.spaceX + self.picX) # thumbnails in X
		self.thumbsY = size_h / (self.spaceY + self.picY) # thumbnails in Y
		self.thumbsC = self.thumbsX * self.thumbsY # all thumbnails

		self.positionlist = []
		skincontent = ""

		posX = -1
		for x in range(self.thumbsC):
			posY = x / self.thumbsX
			posX += 1
			if posX >= self.thumbsX:
				posX = 0

			absX = self.spaceX + (posX*(self.spaceX + self.picX))
			absY = self.spaceY + (posY*(self.spaceY + self.picY))
			self.positionlist.append((absX, absY))
			skincontent += "<widget source=\"label" + str(x) + "\" render=\"Label\" position=\"" + str(absX+5) + "," + str(absY+self.picY-textsize) + "\" size=\"" + str(self.picX - 10) + ","  + str(textsize+2) + "\" font=\"Regular;" + str(textsize) + "\" zPosition=\"2\" transparent=\"1\" noWrap=\"1\" foregroundColor=\"" + self.textcolor + "\" />"
			skincontent += "<widget name=\"thumb" + str(x) + "\" position=\"" + str(absX+5)+ "," + str(absY+5) + "\" size=\"" + str(self.picX -10) + "," + str(self.picY - (textsize*2)) + "\" zPosition=\"2\" transparent=\"1\" />"

		# Screen, backgroundlabel and MovingPixmap
		self.skin = "<screen position=\"0,0\" size=\"" + str(size_w) + "," + str(size_h) + "\" flags=\"wfNoBorder\" > \
			<eLabel position=\"0,0\" zPosition=\"0\" size=\""+ str(size_w) + "," + str(size_h) + "\" backgroundColor=\"" + self.color + "\" /> \
			<widget name=\"frame\" position=\"35,30\" size=\"" + str(self.picX)+ "," + str(self.picX) + "\" pixmap=\"pic_frame.png\" zPosition=\"1\" alphatest=\"on\" />"  + skincontent + "/></screen>"
		#print self.skin
		Screen.__init__(self, session)

		self["actions"] = ActionMap(["OkCancelActions", "ColorActions", "DirectionActions", "MovieSelectionActions", "EPGSelectActions"],
		{
			"cancel": self.Exit,
			"ok": self.KeyOk,
			"left": self.key_left,
			"right": self.key_right,
			"up": self.key_up,
			"down": self.key_down,
			"showEventInfo": self.StartExif,
			"input_date_time": self.menu_pressed,
		}, -1)

		self["frame"] = MovingPixmap()
		for x in range(self.thumbsC):
			self["label"+str(x)] = StaticText()
			self["thumb"+str(x)] = Pixmap()

		self.Thumbnaillist = []
		self.filelist = []
		self.currPage = -1
		self.dirlistcount = 0
		self.path = path
		index = 0
		framePos = 0
		Page = 0
		for x in piclist:
			if x[0][1] == False:
				self.filelist.append((index, framePos, Page, x[0][0],  path + x[0][0]))
				index += 1
				framePos += 1
				if framePos > (self.thumbsC -1):
					framePos = 0
					Page += 1
			else:
				self.dirlistcount += 1

		self.maxentry = len(self.filelist)-1
		self.index = lastindex - self.dirlistcount
		if self.index < 0:
			self.index = 0

		self.picload = ePicLoad()
		self.picload_conn = self.picload.PictureData.connect(self.showPic)

		self.onLayoutFinish.append(self.setPicloadConf)

		self.ThumbTimer = eTimer()
		self.ThumbTimer_conn = self.ThumbTimer.timeout.connect(self.showPic)

	def setPicloadConf(self):
		sc = getScale()
		self._aspectRatio = eSize(sc[0], sc[1])
		self._scaleSize = self["thumb0"].instance.size()
		self.picload.setPara([self._scaleSize.width(), self._scaleSize.height(), sc[0], sc[1], config.pic2.cache.value, int(config.pic2.resize.value), self.color])
		self.paintFrame()

	def paintFrame(self):
		#print "PicturePlayer2]index=" + str(self.index)
		if self.maxentry < self.index or self.index < 0:
			return

		pos = self.positionlist[self.filelist[self.index][T_FRAME_POS]]
		self["frame"].moveTo( pos[0], pos[1], 1)
		self["frame"].startMoving()

		if self.currPage != self.filelist[self.index][T_PAGE]:
			self.currPage = self.filelist[self.index][T_PAGE]
			self.newPage()

	def newPage(self):
		self.Thumbnaillist = []
		#clear Labels and Thumbnail
		for x in range(self.thumbsC):
			self["label"+str(x)].setText("")
			self["thumb"+str(x)].hide()
		#paint Labels and fill Thumbnail-List
		for x in self.filelist:
			if x[T_PAGE] == self.currPage:
				self["label"+str(x[T_FRAME_POS])].setText("(" + str(x[T_INDEX]+1) + ") " + x[T_NAME])
				self.Thumbnaillist.append([0, x[T_FRAME_POS], x[T_FULL]])

		#paint Thumbnail start
		self.showPic()

	def showPic(self, picInfo=""):
		for x in range(len(self.Thumbnaillist)):
			if self.Thumbnaillist[x][0] == 0:
				if fileExists(self.Thumbnaillist[x][2]):
					thumb, isok=resolveThumbOutPath(self.Thumbnaillist[x][2])# check for rotated and parsed Image
					if not isok:
						thumb=self.Thumbnaillist[x][2]
					#print 'PicturePlayer2]showPic]->%s'  %thumb
					if self.picload.getThumbnail(thumb) == 1: #zu tun probier noch mal
						self.ThumbTimer.start(config.pic2.thumbDelay.value, True)
					else:
						self.Thumbnaillist[x][0] = 1
					break
			elif self.Thumbnaillist[x][0] == 1:
				self.Thumbnaillist[x][0] = 2
				ptr = self.picload.getData()
				if ptr != None:
					setPixmap(self["thumb" + str(self.Thumbnaillist[x][1])], ptr, self._scaleSize, self._aspectRatio)
					self["thumb" + str(self.Thumbnaillist[x][1])].show()

	def key_left(self):
		self.index -= 1
		if self.index < 0:
			self.index = self.maxentry
		self.paintFrame()

	def key_right(self):
		self.index += 1
		if self.index > self.maxentry:
			self.index = 0
		self.paintFrame()

	def key_up(self):
		self.index -= self.thumbsX
		if self.index < 0:
			self.index =self.maxentry
		self.paintFrame()

	def key_down(self):
		self.index += self.thumbsX
		if self.index > self.maxentry:
			self.index = 0
		self.paintFrame()

	def StartExif(self):
		if self.maxentry < 0:
			return
		self.session.open(Pic_Exif2, self.picload.getInfo(self.filelist[self.index][T_FULL]))

	def KeyOk(self):
		if self.maxentry < 0:
			return
		self.old_index = self.index
		self.session.openWithCallback(self.callbackView, Pic_Full_View2, self.filelist, self.index, self.path)

	def callbackView(self, val=0, ischanged=False):
		self.index = val
		if self.old_index != self.index or ischanged:
			self.paintFrame()
		
	def menu_pressed(self):
		if self.maxentry < 0:
			return
		self.old_index = self.index
		todelfn = str(self.filelist[self.index][4])
		LOG('todelfn %s' %todelfn)
		self.session.openWithCallback(boundFunction(self.callbackExecDelete, todelfn), MessageBox,_("Do you realy want to DELETE picture:\n" +todelfn), MessageBox.TYPE_YESNO)
		
	def callbackExecDelete(self, todelfn, answer):
		if answer is True:
			LOG('Delete selected Pic %s' % (todelfn))
			os_remove(todelfn) # remove file
			del self.filelist[self.index]
			self.index -= 1
			if self.index < 0:
				self.index = 0
			self.paintFrame()
			self.newPage()
			
	def Exit(self):
		del self.picload_conn
		del self.picload
		self.close(self.index + self.dirlistcount)

class Pic_Full_View2(Screen):


	if RESOLUTIONx>1800:

		skin = """<screen position="0,0" size="1920,1080" flags="wfNoBorder" title="Pictureplayer2.02 Full Screen">
			<eLabel position="0,0" zPosition="0" size="1920,1080"  backgroundColor="#00000000"/>
			<widget name="pic" position="0,0" size="1920,1080" zPosition="1"/>
			<widget name="picprev" position="0,1160" size="1920,120" transparent="1" zPosition="2" coverSize="178,100" selectedCoverScale="1.2" unselectedCoverDimm="0.7"  coverflowCurrentPosition="960,60" coverflowCurrentXDistance="10" coverflowCurrentCenterDistance="20" style="1" noCoverAvailablePic="~/no_picture.png" />
			<widget name="point" position="10,10" size="30,30" zPosition="2" pixmap="skin_default/icons/record.png" alphatest="on" /> 
			<widget name="play_icon" position="50,10" size="30,30" zPosition="2" pixmap="skin_default/icons/ico_mp_play.png"  alphatest="on" /> 
			<widget source="file" render="Label" position="100,10" size="1500,40" font="Regular;32" halign="left" foregroundColor="#ffffff" zPosition="2" noWrap="1" transparent="1" /></screen>"""

	else:

		skin = """<screen position="0,0" size="1280,720" flags="wfNoBorder" title="Pictureplayer2.02 Full Screen">
			<eLabel position="0,0" zPosition="0" size="1280,720"  backgroundColor="#00000000"/>
			<widget name="pic" position="0,0" size="1280,720" zPosition="1"/>
			<widget name="picprev" position="0,640" size="1280,80" transparent="1" zPosition="2" coverSize="119,67" selectedCoverScale="1.2" unselectedCoverDimm="0.7"  coverflowCurrentPosition="480,40" coverflowCurrentXDistance="10" coverflowCurrentCenterDistance="20" style="1" noCoverAvailablePic="~/no_picture.png" />
			<widget name="point" position="8,8" size="20,200" zPosition="2" pixmap="skin_default/icons/record.png" alphatest="on" /> 
			<widget name="play_icon" position="37,8" size="20,20" zPosition="2" pixmap="skin_default/icons/ico_mp_play.png"  alphatest="on" /> 
			<widget source="file" render="Label" position="67,8" size="1000,27" font="Regular;22" halign="left" foregroundColor="#ffffff" zPosition="2" noWrap="1" transparent="1" /></screen>"""
		

	def __init__(self, session, filelist, index, path):
		
		LOG('Pic_Full_View2->init')

		Screen.__init__(self, session)

		self["actions"] = ActionMap(["OkCancelActions", "ColorActions", "DirectionActions", "MovieSelectionActions"],
		{
			"cancel": self.Exit,
			"ok": self.keyOk,
			"green": self.PlayPause,
			"yellow": self.PlayPause,
			"blue": self.nextPic,
			"red": self.prevPic,
			"left": self.prevPic,
			"right": self.nextPic,
			"showEventInfo": self.StartExif,
		}, -1)

		self.keylock=False
		self["point"] = Pixmap()
		self["pic"]= VideoDBPictureBox()
		self['picprev'] = CoverCollection()
		self['picprev'].hide()
		self["play_icon"] = Pixmap()
		self["file"] = StaticText(_("please wait, loading picture..."))
		self.setFileList(filelist, index, path)
		self.picload=ePicLoad()
		self.index=index

		self.slideTimer = eTimer()
		self.slideTimer_conn = self.slideTimer.timeout.connect(self.slidePic)
		
		self.onLayoutFinish.append(self._onLayoutFinished)

	def setFileList(self, filelist, index, path):
		LOG('Pic_Full_View2->setFileList')
		self.picReady = False
		self.old_index = 0
		self.filelist = []
		self.lastindex = index
		self.dirlistcount = 0
		self.ischanged=False

		for x in filelist:
			if len(filelist[0]) == 3: #orig. filelist
				if x[0][1] == False:
					self.filelist.append(path + x[0][0])
				else:
					self.dirlistcount += 1
			elif len(filelist[0]) == 2: #scanlist
				if x[0][1] == False:
					self.filelist.append(x[0][0])
				else:
					self.dirlistcount += 1
			else: # thumbnaillist
				self.filelist.append(x[T_FULL])

		self.maxentry = len(self.filelist)-1
		self.index = index - self.dirlistcount
		if self.index < 0:
			self.index = 0

	def errorPicJob(self, result='---'):
		LOG('errorPicJob]result->%s' %result)
		self.keylock=False

	def _onLayoutFinished(self):
		LOG('Pic_Full_View2->_onLayoutFinished, set transitionDuration')
		self["pic"].setTransitionDuration(280)
		if self.maxentry >= 0:
			self.setPicloadConf()

	def setPicloadConf(self):
		self["play_icon"].hide()
		if not config.pic2.infoline.value:
			self["file"].setText("")
		self.start_decode()

	def start_decode(self):
		self.keylock=True
		try:
			exiflist=self.picload.getInfo(self.filelist[self.index])
			if len(exiflist)>=8:
				LOG('start_decode]Orientation->%s' %str(exiflist[7]))
				orientation=exiflist[7]
			else:
				LOG('start_decode]not Orientation found on exifdata->%s' %str(exiflist))
				orientation=''
		except Exception, e:
			LOG('start_decode]exception->%s' %e)
			self.keylock=False
			return

		picfilename=self.filelist[self.index]
		picpathout, isok=resolveThumbOutPath(picfilename)
		if not isok:
			if orientation in('Top-Left', 'Left-Bottom', 'Right-Top', ''):
				self["file"].setText('Load Pic...')
			else:
				self["file"].setText('Rotate and load Pic...')
			deferToThread(fixAspectRatio, picfilename, picpathout).addCallback(self.decodefinish,).addErrback(self.errorPicJob,)
		else:
			self.decodefinish(picpathout)
			
	def decodefinish(self, picname):
		self.keylock=False
		self["pic"].setPicture(picname)
		try:
			text = self.filelist[self.index]
			text = "(" + str(self.index+1) + "/" + str(self.maxentry+1) + ") " + text.split('/')[-1]
		except Exception, e:
			LOG('decodefinish]exception set text----->%s' %e)
			text = ""
		if not config.pic2.infoline.value:	
			self["file"].setText("")
		else:
			self["file"].setText(text)
		self["point"].hide()
		if self.slideTimer.isActive():# slideshow is running
			LOG('decodefinish]slideshow is running, check to preload next pic index')
			index=self.index+1
			if self.maxentry<index:
				index=0
			picfilename=self.filelist[index]
			picpathout, isok=resolveThumbOutPath(picfilename)
			if not isok:
				LOG('decodefinish]preload next pic now index-->%s' %index)
				deferToThread(fixAspectRatio, picfilename, picpathout).addCallback(self.preloadNextPicFinish,).addErrback(self.errorPicJob,)
			else:
				LOG('decodefinish]next pic is ok, done index-->%s' %index)

	def preloadNextPicFinish(self, result):
		LOG('decodefinish]preload next pic done result-->%s' %result)
		self.keylock=False

	def slidePic(self):
		print "PicturePlayer2]slide to next Picture index=" + str(self.index)
		if config.pic2.loop.value==False and self.index == self.maxentry:
			self.PlayPause()
		self.nextPic()

	def PlayPause(self):
		if self.slideTimer.isActive():
			self.slideTimer.stop()
			self["play_icon"].hide()
			self["pic"].setTransitionDuration(350)
		else:
			self.slideTimer.start(config.pic2.slidetime.value*1000)
			self["play_icon"].show()
			self["pic"].setTransitionDuration(config.pic2.fadeDelay.value)
			self.nextPic()

	def prevPic(self):
		if self.keylock:
			return
		self["file"].setText(_("please wait, loading picture..."))
		self["point"].show()
		self.index -= 1
		if self.index < 0:
			self.index = self.maxentry
		self.start_decode()		

	def nextPic(self):
		if self.keylock:
			return
		#self['picprev'].show()
		self["point"].show()
		self.index += 1
		if self.index > self.maxentry:
			self.index = 0
		self.start_decode()		

	def keyOk(self):
		if self.keylock:
			return
		todelfn=self.filelist[self.index]
		self.session.openWithCallback(boundFunction(self.callbackDelete, todelfn), MessageBox,_("Do you realy want DELETE picture:\n" +todelfn), MessageBox.TYPE_YESNO)
		
	def callbackDelete(self, todelfn, res):
		if res:
			LOG('Delete now-->%s' %todelfn)
			self.ischanged=True
			os_remove(todelfn)
			self.filelist.pop(self.index)
		self.maxentry-=1
		if self.maxentry<=0:
			self.Exit()
		else:
			self.index-=1
			if self.index<0:
				self.index=0
			self.nextPic()		
		
	def StartExif(self):
		if self.maxentry < 0:
			return
		self.session.open(Pic_Exif2, self.picload.getInfo(self.filelist[self.lastindex]))

	def Exit(self):
		self.slideTimer.stop()
		del self.picload
		self.close(self.index + self.dirlistcount,)
